/*
 *  $Id: ports.cc,v 1.1.1.1 1998/01/11 00:00:37 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of KCemu, the KC 85/4 Emulator.
 */

#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>

#include "kc/kc.h"
#include "kc/ctc.h"
#include "kc/ports.h"
#include "kc/memory.h"

Ports::Ports(void)
{
    int a;
    
    for (a = 0;a < NR_PORTS;a++) {
	inout[a] = 0;
    }
    inout[0x88] = 0x0f;
}

byte_t Ports::in(word_t addr)
{
    int val;

    addr &= 0xff;
    val = inout[addr];

    /*
    if (addr == 0x84) {
      printf("Ports::in():  %04x -> %02x\n", addr, inout[addr]);
    }
    */
    switch (addr) {
    case 0x8c:
	val = ctc->in(0);
	//cout << "IN  CTC Channel 0: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	break;
    case 0x8d:
	val = ctc->in(1);
	//cout << "IN  CTC Channel 1: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	break;
    case 0x8e:
	val = ctc->in(2);
	//cout << "IN  CTC Channel 2: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	break;
    case 0x8f:
	val = ctc->in(3);
	//cout << "IN  CTC Channel 3: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	break;
    }
    return val;
}

void Ports::out(word_t addr, byte_t val)
{
    byte_t a = addr & 0xff;

    /*
    if (addr == 0x84) {
      printf("Ports::out(): %04x <- %02x\n", addr, val);
    }
    */
    switch (a) {
    case 0x84: change_0x84(inout[0x84] ^ val, val); break;
    case 0x86: change_0x86(inout[0x86] ^ val, val); break;
    case 0x88: change_0x88(inout[0x88] ^ val, val); break;
    case 0x89:
        //cout << "OUT PIO B Data Word: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	break;
    case 0x8a:
	//cout << "OUT PIO A Control Word: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	break;
    case 0x8b:
	//cout << "OUT PIO B Control Word: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	break;
    case 0x8c:
	//cout << "OUT CTC Channel 0: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	ctc->out(0, val);
	return;
    case 0x8d:
	//cout << "OUT CTC Channel 1: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	ctc->out(1, val);
	return;
    case 0x8e:
	//cout << "OUT CTC Channel 2: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	ctc->out(2, val);
	return;
    case 0x8f:
	//cout << "OUT CTC Channel 3: " << hex << setfill('0') << setw(2)
	//     << (int)val << "\n";
	ctc->out(3, val);
	return;
    }
    inout[addr & 0xff] = val;
}

void Ports::change_0x84(byte_t changed, byte_t val)
{
    if (changed & 0x01) {
	/* display screen 0 or 1 */
	//fprintf(stderr, "Ports 0x84: display screen %d\n", val & 1);
    }
    if (changed & 0x02) {
	//fprintf(stderr, "Ports 0x84: pixel/ color %d\n", (val >> 1) & 1);
	memory->enableCOLOR(val & 0x02);
    }
    if (changed & 0x04) {
	//fprintf(stderr, "Ports 0x84: access screen %d\n", (val >> 2) & 1);
	memory->enableSCREEN_0(val & 0x04);
    }
    if (changed & 0x08) {
	//fprintf(stderr, "Ports 0x84: high resolution %d\n", (val >> 3) & 1);
    }
    if (changed & 0x10) {
	//fprintf(stderr, "Ports 0x84: access RAM8 block %d\n", (val >> 4) & 1);
    }
    if (changed & 0x20) {
	//fprintf(stderr, "Ports 0x84: RAM8 on/off %d\n", (val >> 5) & 1);
	memory->enableRAM8(val & 0x20);
    }
    if (changed & 0x40) {
	//fprintf(stderr, "Ports 0x84: reserved 0x40 %d\n", (val >> 6) & 1);
    }
    if (changed & 0x80) {
	//fprintf(stderr, "Ports 0x84: reserved 0x80 %d\n", (val >> 7) & 1);
    }
}

void Ports::change_0x86(byte_t changed, byte_t val)
{
    if (changed & 0x01) {
	//fprintf(stderr, "Ports 0x86: RAM 4 %d\n", val & 1);
	memory->enableRAM_4(val & 0x01);
    }
    if (changed & 0x02) {
	//fprintf(stderr, "Ports 0x86: write protect RAM 4 %d\n", (val >> 1) & 1);
	memory->protectRAM_4(val & 0x02);
    }
    if (changed & 0x04) {
	//fprintf(stderr, "Ports 0x86: unused 0x04 %d\n", (val >> 2) & 1);
    }
    if (changed & 0x08) {
	//fprintf(stderr, "Ports 0x86: unused 0x08 %d\n", (val >> 3) & 1);
    }
    if (changed & 0x10) {
	//fprintf(stderr, "Ports 0x86: unused 0x10 %d\n", (val >> 4) & 1);
    }
    if (changed & 0x20) {
	//fprintf(stderr, "Ports 0x86: reserved 0x20 %d\n", (val >> 5) & 1);
    }
    if (changed & 0x40) {
	//fprintf(stderr, "Ports 0x86: reserved 0x40 %d\n", (val >> 6) & 1);
    }
    if (changed & 0x80) {
	//fprintf(stderr, "Ports 0x86: CAOS C %d\n", (val >> 7) & 1);
	memory->enableCAOS_C(val & 0x80);
    }
}

void Ports::change_0x88(byte_t changed, byte_t val)
{
    if (changed & 0x01) {
	//fprintf(stderr, "Ports 0x88: CAOS ROM E %d\n", val & 1);
	memory->enableCAOS_E(val & 0x01);
    }
    if (changed & 0x02) {
	//fprintf(stderr, "Ports 0x88: RAM 0 %d\n", (val >> 1) & 1);
	memory->enableRAM_0(val & 0x02);
    }
    if (changed & 0x04) {
	//fprintf(stderr, "Ports 0x88: IRM %d\n", (val >> 2) & 1);
	memory->enableIRM(val & 0x04);
    }
    if (changed & 0x08) {
	//fprintf(stderr, "Ports 0x88: write protect RAM 0 %d\n", (val >> 3) & 1);
	memory->protectRAM_0(val & 0x08);
    }
    if (changed & 0x10) {
	//fprintf(stderr, "Ports 0x88: K OUT (unused) %d\n", (val >> 4) & 1);
    }
    if (changed & 0x20) {
	//fprintf(stderr, "Ports 0x88: LED %d\n", (val >> 5) & 1);
    }
    if (changed & 0x40) {
	//fprintf(stderr, "Ports 0x88: Tape Power %d\n", (val >> 6) & 1);
    }
    if (changed & 0x80) {
	//fprintf(stderr, "Ports 0x88: ROM C (BASIC) %d\n", (val >> 7) & 1);
	memory->enableBASIC_C(val & 0x80);
    }
}

