/*
 *  $Id: z80.cc,v 1.1.1.1 1998/01/11 00:00:37 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of KCemu, the KC 85/4 Emulator.
 */

#include <stdio.h>
#include <signal.h>
#include <iostream.h>
#include <iomanip.h>
#include <sys/time.h>

#include "kc/kc.h"
#include "kc/z80.h"
#include "kc/ctc.h"
#include "kc/ports.h"
#include "kc/memory.h"

#include "ui/ui.h"

#define I_PERIOD 1750

static Z80 *self;
static struct timeval tv, otv;

static void signalHandler(int sig);

void WrZ80(word_t Addr, byte_t Value)
{
	//printf("memWrite: %04x, %02x\n", Addr, Value);
	memory->memWrite8(Addr, Value);
}

byte_t RdZ80(word_t Addr)
{
	//printf("memRead:  %04x\n", Addr);
	return memory->memRead8(Addr);
}

void OutZ80(word_t Port, byte_t Value)
{
	ports->out(Port, Value);
}


byte_t InZ80(word_t Port)
{
	return ports->in(Port);
}

void PatchZ80(_Z80 * /*R*/)
{
}

word_t LoopZ80(_Z80 * /*R*/)
{
	byte_t irq;
	long ut;
	long diff;
	static int a = 0;
	static long time = 0;

	if (self->_do_reset || self->_do_power_on || self->_do_quit) {
		return INT_QUIT;
	}

	if (++a == 20) {
		a = 0;
		ui->processEvents();
		ui->update(false);
	}

	gettimeofday(&tv, 0);
	if (tv.tv_sec > otv.tv_sec) tv.tv_usec += 1000000L;
	ut = tv.tv_usec - otv.tv_usec;

#ifdef MSDOS
	diff = (long)((I_PERIOD / 3) - ut);
#endif /* MSDOS */
#ifdef LINUX
	diff = (long)((I_PERIOD / 1.75) - ut);
#endif /* LINUX */
	if ((time += diff) < 0) time = 0;
	if (time > 20000) {
		time = 0;
		struct timeval tv1;
		tv1.tv_sec = 0;
		tv1.tv_usec = 20000;
		select(0, 0, 0, 0, &tv1);
	}
	gettimeofday(&otv, 0);

	if ((irq = ctc->checkInt()) != 0xff) {
		return irq;
	}

	return INT_NONE;
}

Z80::Z80(void)
{
	self = this;
	signal(SIGINT, signalHandler);

        _do_quit = false;
	_do_reset = false;
	_do_power_on = true;

	regs.IPeriod = I_PERIOD;
	regs.TrapBadOps = 1;
	regs.Trace = 0;
	regs.Trap = 0xffff;
}

void Z80::run(void)
{
        while (_do_reset || _do_power_on) {
                ResetZ80(&regs);
                if (_do_reset)    regs.PC.W = 0xe000;
                if (_do_power_on) regs.PC.W = 0xf000;
                _do_reset = false;
                _do_power_on = false;
                gettimeofday(&otv, 0);
                RunZ80(&regs);
        }
}

void Z80::quit(void)
{
        _do_quit = true;
}

void Z80::reset(void)
{
        _do_reset = true;
}

void Z80::powerOn(void)
{
        _do_power_on = true;
}

void Z80::trace(byte_t level)
{
	regs.Trace = level;
}

void Z80::triggerInt(int vector)
{
	IntZ80(&regs, vector);
}

static void signalHandler(int sig) 
{
	cout << "\n *** signal caught (" << sig << ") ***\n\n";
	signal(SIGINT, signalHandler);
	self->trace(1);
}
