/*
 *  $Id: ui_xwin.cc,v 1.1.1.1 1998/01/11 00:00:37 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of KCemu, the KC 85/4 Emulator.
 */

#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <iostream.h>

#include "kc/kc.h"
#include "kc/z80.h"
#include "kc/ctc.h"
#include "ui/ui_xwin.h"

/*
 *   0 schwarz
 *   1 blau
 *   2 rot
 *   3 magenta
 *   4 grn
 *   5 trkis
 *   6 gelb
 *   7 wei
 *   8 dunkelgrau
 *   9 lila
 *  10 orange
 *  11 weinrot
 *  12 hellgrn
 *  13 hellblau
 *  14 dunkelgrn
 *  15 wei
 */

UI_XWin::UI_XWin(void)
{
    int a;
    Visual *vis;
    XColor screen_color, exact_color;
    char *color_names[] = {
	"rgb:00/00/00",
	"rgb:00/00/d0",
	"rgb:d0/00/00",
	"rgb:d0/00/d0",
	"rgb:00/d0/00",
	"rgb:00/d0/d0",
	"rgb:d0/00/00",
	"rgb:d0/d0/d0",

	"rgb:00/00/00",
	"rgb:60/00/a0",
	"rgb:a0/60/00",
	"rgb:a0/00/60",
	"rgb:00/a0/60",
	"rgb:00/60/a0",
	"rgb:30/a0/30",
	"rgb:d0/d0/d0",

	"rgb:00/00/00",
	"rgb:00/00/a0",
	"rgb:a0/00/00",
	"rgb:a0/00/a0",
	"rgb:00/a0/00",
	"rgb:00/a0/a0",
	"rgb:a0/a0/00",
	"rgb:a0/a0/a0"
    };

    _shift = 0;
    _dirty = true;

    _disp = XOpenDisplay(NULL);
    if (_disp == NULL) {
	cerr << "can't open display\n";
	exit(1);
    }
    _screen = DefaultScreen(_disp);
    _w = XCreateSimpleWindow(_disp, RootWindow(_disp, _screen), 0, 0,
			    320, 256, 2, BlackPixel(_disp, _screen),
			    WhitePixel(_disp, _screen));
    XStoreName(_disp, _w, WINDOW_NAME);
    XSetIconName(_disp, _w, ICON_NAME);
    XSelectInput(_disp, _w, ExposureMask | KeyPressMask | KeyReleaseMask);
    XMapRaised(_disp, _w);

    _wm_delete_window = XInternAtom(_disp, "WM_DELETE_WINDOW", False);
    XSetWMProtocols (_disp, _w, &_wm_delete_window, 1);

    _black_gc = XCreateGC(_disp, _w, 0, NULL);
    _white_gc = XCreateGC(_disp, _w, 0, NULL);
    XSetForeground(_disp, _black_gc, BlackPixel(_disp, _screen));
    XSetForeground(_disp, _white_gc, WhitePixel(_disp, _screen));

    if (XMatchVisualInfo(_disp, _screen, 24, TrueColor, &_visualInfo)) {
    } else if (XMatchVisualInfo(_disp, _screen, 16, TrueColor, &_visualInfo)) {
    } else if (XMatchVisualInfo(_disp, _screen, 15, TrueColor, &_visualInfo)) {
    } else if (XMatchVisualInfo(_disp, _screen, 8, PseudoColor, &_visualInfo)){
    } else if (XMatchVisualInfo(_disp, _screen, 8, GrayScale, &_visualInfo)) {
    } else {
        cerr << "Can't obtain appropriate X visual\n";
        exit(1);
    }
    vis = _visualInfo.visual;
    _bitdepth = _visualInfo.depth;
    _pixbytes = (_bitdepth == 24 || _bitdepth == 32 ? 4 :
                _bitdepth == 12 || _bitdepth == 16 ? 2 : 1);
    _img = XShmCreateImage(_disp, vis, _bitdepth, ZPixmap, 0, &_shminfo,
			   320, 256);
    _buflen = 256 * _img->bytes_per_line;
    _shminfo.shmid = shmget(IPC_PRIVATE, _buflen, IPC_CREAT | 0777);
    _shminfo.shmaddr = _img->data = _bufmem =(char *)shmat(_shminfo.shmid,0,0);
    _shminfo.readOnly = False;
    XShmAttach(_disp, &_shminfo);
    XSync(_disp, False);
    shmctl(_shminfo.shmid, IPC_RMID, 0);

    _pix_mem = new char[256 * 40];
    _col_mem = new char[256 * 40];

    for (a = 0;a < 24;a++) {
	cout << "allocating color '" << color_names[a] << "' ";
	if (XAllocNamedColor(_disp, DefaultColormap(_disp, _screen),
			     color_names[a],
			     &screen_color, &exact_color) == 0) {
	    cout << "failed!\n";
	    exit(0);
	} else {
	    cout << "ok.\n";
	}
	_col[a] = screen_color.pixel;
    }
}

void UI_XWin::update(bool force_update)
{
    if (_dirty || force_update) {
	_dirty = false;
	XShmPutImage(_disp, _w, _white_gc, _img, 0, 0, 0, 0, 320, 256, 0);
	XSync(_disp, False);
    }
}

void UI_XWin::putPixel(int x, int y, char val)
{
    int a, c;
    long fg, bg;

    a = (x << 5) | y;
    _pix_mem[a] = val;
    c = _col_mem[a];
    bg = _col[(c & 7) | 0x10];
    fg = _col[(c >> 3) & 15];

    for (a = 0;a < 8;a++) { 
	XPutPixel(_img, x + a, y, (val & 0x80) ? fg : bg);
	val <<= 1;
    }

    _dirty = true;
}

void UI_XWin::putColor(int x, int y, char val)
{
    int a, c;
    long fg, bg;

    a = (x << 5) | y;
    _col_mem[a] = val;
    bg = _col[(val & 7) | 0x10];
    fg = _col[(val >> 3) & 15];
    c = _pix_mem[a];

    for (a = 0;a < 8;a++) {
	XPutPixel(_img, x + a, y, (c & 0x80) ? fg : bg);
	c <<= 1;
    }

    _dirty = true;
}

void UI_XWin::handleKeyPress(XEvent *event)
{

    int count;
    KeySym key_sym;
    char buffer[20];
    int c = 0;
    XComposeStatus compose;

    count = XLookupString(&event->xkey, buffer, 20, &key_sym, &compose);

    switch (key_sym) {
    case XK_Shift_R:
    case XK_Shift_L:    _shift = 1;
	                return;
    case XK_Left:       c = 0x100; break;
    case XK_Right:      c = 0x101; break;
    case XK_Up:         c = 0x102; break;
    case XK_Down:       c = 0x103; break;
    case XK_Home:       if (!_shift) c = 0x104; else c = 0x105; break;
    case XK_BackSpace:	c = 0x106; break;
    case XK_Escape:	c = 0x1b; break;
    case XK_Delete:	if (!_shift) c = 0x1f; else c = 0x02; break;
    case XK_F1:		if (!_shift) c = 0xf1; else c = 0xf7; break;
    case XK_F2:		if (!_shift) c = 0xf2; else c = 0xf8; break;
    case XK_F3:		if (!_shift) c = 0xf3; else c = 0xf9; break;
    case XK_F4:		if (!_shift) c = 0xf4; else c = 0xfa; break;
    case XK_F5:		if (!_shift) c = 0xf5; else c = 0xfb; break;
    case XK_F6:		if (!_shift) c = 0xf6; else c = 0xfc; break;
    case XK_Insert:	if (!_shift) c = 0x1a; else c = 0x14; break;
    case XK_F12:	c = 0x03;
	                break;
    case XK_F9:         //theProcessor.setTraceLevel(1);
	                return;
    default:		c = buffer[0];
	                break;

    case XK_F10:	fprintf(stderr, "*** RESET ***\n");
	                z80->reset();
			return;
    case XK_F11:	fprintf(stderr, "*** POWER ON ***\n");
			z80->powerOn();
	                //z80->triggerInt(0xe6);
			return;
    case XK_Find:       z80->quit();
			return;
    }

    ctc->keyPressed(c);

    /*
    memory->memWrite8(0x1fd, c);
    memory->memWrite8(0x1f8, 0x01);
    */
}

void UI_XWin::handleKeyRelease(XEvent *event)
{
    KeySym key_sym;

    key_sym = XKeycodeToKeysym(_disp, event->xkey.keycode, 0);
    switch (key_sym) {
    case XK_Shift_R:
    case XK_Shift_L: _shift = 0;
                     return;
    }
    //printf("KeyRelease (0x%08lx %c)\n", key_sym, key_sym);
}

void UI_XWin::processEvents(void)
{
    XEvent event;

    if (XPending(_disp)) {
	XNextEvent(_disp, &event);
	switch (event.type) {
	case KeyPress:
	    handleKeyPress(&event);
	    break;
	case KeyRelease:
	    handleKeyRelease(&event);
	    break;
	case Expose:
	    printf("Expose\n");
	    _dirty = true;
	    break;
	default:
	    printf("unknown X event\n");
	    exit(0);
	    break;
	}
    }
}


/*
int main(int argc, char **argv)
{
    int a, arg;
    
    
    init();
    theProcessor.opsInit();
    theProcessor.jumpTo(0xf000);

    arg = 1;
    while (arg < argc) {
	if (argv[arg][0] != '-') {
	    break;
	} else if (strcmp("-prof", argv[arg]) == 0) {
	    theProcessor.setProfiling(1);
	    arg++;
	} else if (strcmp("-d", argv[arg]) == 0) {
	    theProcessor.setTraceLevel(1);
	    arg++;
	} else if (strcmp("-u", argv[arg]) == 0) {
	    arg++;
	    update = strtol(argv[arg], NULL, 0);
	    arg++;
	} else if (strcmp("-br", argv[arg]) == 0) {
	    arg++;
	    theProcessor.addBreakpoint(strtol(argv[arg], NULL, 0), BPT_READ);
	    arg++;
	} else if (strcmp("-bw", argv[arg]) == 0) {
	    arg++;
	    theProcessor.addBreakpoint(strtol(argv[arg], NULL, 0), BPT_WRITE);
	    arg++;
	} else if (strcmp("-brw", argv[arg]) == 0) {
	    arg++;
	    theProcessor.addBreakpoint(strtol(argv[arg], NULL, 0),
				       BPT_READ_WRITE);
	    arg++;
	} else if (strcmp("-be", argv[arg]) == 0) {
	    arg++;
	    theProcessor.addBreakpoint(strtol(argv[arg], NULL, 0), BPT_EXEC);
	    arg++;
	} else if (strcmp("-ba", argv[arg]) == 0) {
	    arg++;
	    theProcessor.addBreakpoint(strtol(argv[arg], NULL, 0), BPT_ANY);
	    arg++;
	} else if (strcmp("-pc", argv[arg]) == 0) {
	    arg++;
	    theProcessor.jumpTo(strtol(argv[arg], NULL, 0));
	    arg++;
	} else {
	    printf("unknown option '%s' ignored!\n", argv[arg++]);
	}
    }

    XSync(disp, False);
    printf("Update: %lu\n", update);
    theProcessor.profilingStart();
    while (1) {
	for (a = 0;a < update;a++) {
	    theProcessor.execute();
	}
	if (update_display) updateDisplay();
    }

    return 0;
}
*/
